package be.iminds.ilabt.jfed.testing.base;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

/**
 * LegacyApiTestConfigEditor
 */
public class LegacyApiTestConfigEditor implements ApiTestConfigEditor<LegacyApiTestConfig> {
    @Nonnull private final LegacyApiTestMetaData legacyApiTestMetaData;
    @Nonnull private Properties res = new Properties();

    public LegacyApiTestConfigEditor(@Nonnull LegacyApiTestMetaData legacyApiTestMetaData) {
        this.legacyApiTestMetaData = legacyApiTestMetaData;
    }

    @Nonnull
    @Override
    public List<ApiTestConfigParameter> getParameters() {
        List<ApiTestConfigParameter> res = new ArrayList<>();
        for (String reqKey : legacyApiTestMetaData.getReqConfKeys()) {
            res.add(new ApiTestConfigParameter(
                    Collections.singletonList(reqKey),
                    "no description",
                    String.class,
                    null,
                    false,
                    null
            ));
        }
        for (String reqKey : legacyApiTestMetaData.getOptConfKeys()) {
            res.add(new ApiTestConfigParameter(
                    Collections.singletonList(reqKey),
                    "no description",
                    String.class,
                    null,
                    true,
                    null
            ));
        }
        return res;
    }

    @Override
    public void setParameter(@Nonnull ApiTestConfigParameter parameter, @Nullable Object value) {
        if (value == null) {
            res.remove(parameter.getLastName());
        } else {
            res.setProperty(parameter.getLastName(), value + "");
        }
    }

    @Nullable
    @Override
    public Object getParameter(@Nonnull ApiTestConfigParameter parameter) {
        return res.getProperty(parameter.getLastName());
    }

    @Override
    public void setConfig(@Nonnull LegacyApiTestConfig apiTestConfig) {
        res = apiTestConfig.getProperties();
    }

    @Nonnull
    @Override
    public LegacyApiTestConfig build() {
        return new LegacyApiTestConfig(res, legacyApiTestMetaData);
    }
}
