package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.view;

import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogItem;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogModel;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecStateItem;
import javafx.collections.ListChangeListener;
import javafx.scene.control.Button;
import javafx.scene.control.Control;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

class ESpecStateButton extends VBox {
    public static interface ClickedHandler {
        public void onStateClicked(ESpecStateItem item);
    }

    private final ESpecStateItem eSpecStateItem;

    private final Button mainButton;
    private final HBox extraButtonsBox;

    @Nullable private ClickedHandler onClickedHandler;

    public ESpecStateButton(@Nonnull ESpecStateItem eSpecStateItem) {
        this.eSpecStateItem = eSpecStateItem;

        this.mainButton = createBaseButton(eSpecStateItem);
        this.mainButton.getStyleClass().add("without-substatus");
        this.extraButtonsBox = new HBox();

        extraButtonsBox.setMaxWidth(Double.MAX_VALUE);
        VBox.setVgrow(mainButton, Priority.ALWAYS);
        VBox.setVgrow(extraButtonsBox, Priority.ALWAYS);

        getChildren().addAll(mainButton, extraButtonsBox);

        eSpecStateItem.getChildren().addListener(
                (ListChangeListener<ESpecStateItem>) (c) -> {
                    while (c.next()) {
                        if (c.wasAdded()) {
                            for (ESpecStateItem i : c.getAddedSubList())
                                addSubButton(i);
                        }
                    }
                });

        eSpecStateItem.getChildren().addListener(
                (ListChangeListener<ESpecStateItem>) (change) -> {
                    if (!eSpecStateItem.getChildren().isEmpty()) {
                        mainButton.getStyleClass().remove("without-substatus");
                        mainButton.getStyleClass().add("with-substatus");
                    }
                }
        );

//        eSpecStateItem.nameProperty().addListener((o, oldVal, newVal) -> updateName());
//        eSpecStateItem.getChildren().addListener(
//                (ListChangeListener<ESpecStateItem>) (change) -> updateName());
//        updateName();
    }

    public void setOnClickedHandler(@Nullable ClickedHandler onClickedHandler) {
        this.onClickedHandler = onClickedHandler;
    }
    public void fireOnClickedHandler(@Nonnull ESpecStateItem eSpecStateItem) {
        if (onClickedHandler != null) {
            onClickedHandler.onStateClicked(eSpecStateItem);
        }
    }

    public Button createBaseButton(ESpecStateItem eSpecStateItem) {
        Button res = new Button();
        res.getStylesheets().add(getClass().getResource("espec-state-item.css").toExternalForm());
        res.getStyleClass().addAll("especState");

        res.textProperty().bind(eSpecStateItem.humanNameProperty());

//        res.setText(eSpecStateItem.getHumanName()+" ("+eSpecStateItem.getLogItems().size()+")");
//        eSpecStateItem.getLogItems().addListener(
//                (ListChangeListener<ESpecLogItem>) (change) -> {
//                    res.setText(eSpecStateItem.getHumanName()+" ("+eSpecStateItem.getLogItems().size()+")");
//                });

        eSpecStateItem.statusProperty().addListener(
                (o, oldVal, newVal) -> updateStatus(newVal, res)
        );
        updateStatus(eSpecStateItem.getStatus(), res);
        //these buttons should take up all space they get, but not prefer it.
        res.setMaxWidth(Double.MAX_VALUE);
        res.setOnAction((ae) -> fireOnClickedHandler(eSpecStateItem));
        return res;
    }

    public void addSubButton(ESpecStateItem eSpecStateItem) {
        Button subButton = createBaseButton(eSpecStateItem);
        subButton.getStyleClass().add("with-substatus");
        extraButtonsBox.getChildren().add(subButton);
    }

//    public void updateName() {
//        String newText = eSpecStateItem.getHumanName();
//        if (!eSpecStateItem.getChildren().isEmpty()) {
//            newText += eSpecStateItem.getChildren().stream()
//                    .map(ESpecStateItem::getHumanName)
//                    .collect(Collectors.joining(", "));
//            getChildren().add(new Label("Test"));
//        }
//        this.mainButton.setText(newText);
//    }

    public void updateStatus(ESpecStateItem.ESpecStateStatus status, Control target) {
        List<String> oldStyleClasses = new ArrayList(Arrays.asList("future", "running", "fail", "success"));
        List<String> newStyleClasses = new ArrayList();

        switch (status) {
            case FAILURE: {
                oldStyleClasses.remove("fail");
                newStyleClasses.add("fail");
                break;
            }
            case FUTURE: {
                oldStyleClasses.remove("future");
                newStyleClasses.add("future");
                break;
            }
            case SUCCESS: {
                oldStyleClasses.remove("success");
                newStyleClasses.add("success");
                break;
            }
            case RUNNING: {
                oldStyleClasses.remove("running");
                newStyleClasses.add("running");
                break;
            }
        }

        target.getStyleClass().removeAll(oldStyleClasses);
        newStyleClasses.removeAll(target.getStyleClass());
        target.getStyleClass().addAll(newStyleClasses);
    }
//
//    public void updateSize() {
//        Control target = this.mainButton;
//        List<String> newStyleClasses = new ArrayList();
//
//        if (this.eSpecStateItem.getChildren().isEmpty()) {
//            target.getStyleClass().remove("without-substatus");
//            newStyleClasses.add("with-substatus");
//        } else {
//            target.getStyleClass().remove("with-substatus");
//            newStyleClasses.add("without-substatus");
//        }
//
//        //add missing
//        newStyleClasses.removeAll(target.getStyleClass());
//        target.getStyleClass().addAll(newStyleClasses);
//    }
}
